import React, { Component } from 'react';
// import JSONUtil from './jsonUtils';
import './index.css';
let startX = 0;
let maxWidth, minWidth;
let shouldMove = false;
let throttle = function (fn, delay, mustRunDelay) {
  var timer = null;
  var tStart;

  return function () {
    var context = this;
    var args = arguments;
    var tCurr = +new Date();

    clearTimeout(timer);
    if (!tStart) {
      tStart = tCurr;
    }
    if (tCurr - tStart >= mustRunDelay) {
      fn.apply(context, args);
      tStart = tCurr;
    } else {
      timer = setTimeout(function () {
        fn.apply(context, args);
      }, delay);
    }
  };
};

export default class Dialog extends Component {
  constructor (props) {
    super(props);

    this.state = {
      tab: 'headers',
      dialogWidth: 900,
      viewParsed: true
    };

    document.onmousemove = throttle(this.onMousemove.bind(this), 50, 100);
    document.onmouseup = this.onMouseup.bind(this);
  }

  componentWillReceiveProps (nextProps) {
    if (nextProps.requestDetail && this.state.tab == 'response') {
      if (nextProps.requestDetail.id !== this.props.requestDetail.id) {
        let cache = getCacheBody(nextProps.requestDetail.id);
        if (cache) {
          return this.setState({
            responseBody: cache,
            // tab: 'headers',
            viewParsed: true
          });
        }
        if (nextProps.requestDetail.originLength <= 1 * 1024 * 1024) {
          fetch('/fetchresponse?reqId=' + nextProps.requestDetail.id)
              .then(data => data.text())
              .then((body) => {
                cacheBody(nextProps.requestDetail.id, body);
                this.setState({
                  responseBody: body,
                  // tab: 'headers',
                  viewParsed: true
                });
              });
        } else {
          this.setState({
            responseBody: '',
            // tab: 'headers',
            viewParsed: true
          });
        }

      }
    } else {
      this.setState({
        // tab: 'headers',
        viewParsed: true
      });
    }
  }

  componentDidUpdate () {
    // var block = document.getElementById('js-code');
    // var text = block && block.textContent || '';
    // block && text.length < 20 * 1024 && hljs.highlightBlock(block);
  }
  onMousedown (eve) {
    shouldMove = true;
    maxWidth = 1200;
    minWidth = 100;
    startX = eve.clientX;
    eve.preventDefault();
  }

  onMousemove (eve) {
    if (!shouldMove) {
      return;
    }
    let {clientX} = eve;
    let delta = startX - clientX;
    startX = startX - delta;
    let width = parseInt(document.querySelector('.dialog').style.width);
    width += delta;
    width = width < minWidth ? minWidth : width > maxWidth ? maxWidth : width;
    document.querySelector('.dialog').style.width = width + 'px';
  }

  onMouseup () {
    shouldMove = false;
  }
  changeViewJson () {
    let viewParsed = this.state.viewParsed;
    this.setState({viewParsed: !viewParsed});
  }
  render () {
    const { showRequestDetail, requestDetail, onClose } = this.props;
    const tab = this.state.tab;
    let responseBody = this.state.responseBody;

    if (!showRequestDetail) {
      return null;
    }
    const t = requestDetail;

    const content = () => {
      if (tab === 'headers') {
        return <section className='body'>
          <h3 className='header'>General</h3>
          <ul className='list'>
            <li><strong>Request URL:</strong>  {t.url.href}</li>
            <li><strong>Proxy URL:</strong>  {t.newUrl}</li>
            <li><strong>Request Method:</strong> {t.method}</li>
            <li><strong>Status Code:</strong> {t.statusCode}</li>
            <li><strong>Remote Address:</strong> {t.hostname}</li>
          </ul>
          <h3 className='header'>Response Headers</h3>
          <ul className='list'>{parseData(t.resHeaders)}</ul>
          <h3 className='header'>Request Headers</h3>
          <ul className='list'>{parseData(t.headers)}</ul>
          {this.renderParams(t)}
        </section>;
      } else if (tab === 'response') {
        // 获取content-type
        let {resHeaders, fileType} = t;

        let targetURL = t.protocol + '//' + t.hostname + (t.port ? ':' + t.port : '') + t.path;

        if (/^(html|css|javascript|json|text)$/.test(fileType)) {
          if (t.originLength > 1 * 1024 * 1024) {
            return <div style={{padding: '10px'}}>文件内容太长，<a href={targetURL} target='_blank'>点击此处</a>在新窗口中打开。</div>;
          } else {
            let data = responseBody;

            if (fileType === 'json') {
              try {
                data = JSON.stringify(JSON.parse(t.socketData), null, 2);
              } catch (err) {
                // ...
                data = responseBody;
              }
            }
            return (
              <pre className='code' id='js-code'>
                <code className={fileType}>{data}</code>
              </pre>
            );
          }
        } else if (/^(png|jpg|jpeg|gif|ico|svg)$/.test(fileType)) {
          return <div className='content'><img src={targetURL} /></div>;
        } else {
          return <div style={{padding: '10px'}}>暂时不支持此类型文件预览，<a href={targetURL} target='_blank'>点击此处</a>在新窗口中打开。</div>;
        }
      }
    };

    return (
      <div className='dialog' style={{width: this.state.dialogWidth + 'px'}}>
        <div id='spliter'
          onMouseDown={this.onMousedown.bind(this)}
        />
        <header>
          <div className='close' onClick={this.close.bind(this)}>&times;</div>
          <div className={tab === 'headers' ? 'tab active' : 'tab'}
            onClick={this.switchTab.bind(this, 'headers', t.id, t.originLength)}>Headers</div>
          <div className={tab === 'response' ? 'tab active' : 'tab'}
            onClick={this.switchTab.bind(this, 'response', t.id, t.originLength)}>Response</div>
        </header>
        {content()}
      </div>
    );
  }

  renderParams (t) {
    let bodyType = getBodyType(t);
    let method = t.method.toLocaleLowerCase();
    let bodyOrqs = method === 'get' ? t.querystring : t.body;
    let bodyData = getBody(t);
    let body = '';
    let { viewParsed } = this.state;

    if (!bodyType || !bodyData) {
      return null;
    }

    if (bodyType === 'Request Payload' || !viewParsed) {
      body = <pre><code>{viewParsed ? bodyData : bodyOrqs}</code></pre>;
    } else {
      body = <ul className='list'>{bodyData}</ul>;
    }

    return (
      <div>
        <h3 className='header'>
          { bodyType }
          <span
            style={{marginLeft: '20px', fontSize: '12px', color: '#838383'}}
            onClick={this.changeViewJson.bind(this)}
          >
            { viewParsed ? 'view source' : 'view parsed'}
          </span>
        </h3>
        {body}
      </div>
    );
  }

  close (eve) {
    let {onClose} = this.props;

    onClose && onClose();
  }

  switchTab (tab, id, len) {
    let cache = getCacheBody(id);
    if (cache) {
      return this.setState({
        responseBody: cache,
        tab
      });
    }

    if (tab === 'response') {
      if (len <= 1 * 1024 * 1024) {
        fetch('/fetchresponse?reqId=' + id)
            .then(data => data.text())
            .then((body) => {
              cacheBody(id, body);
              this.setState({
                responseBody: body,
                tab
              });
            });
      } else {
        this.setState({
          responseBody: '',
          tab
        })
      }

    } else {
      this.setState({tab});
    }
  }

  showRequestDetail (item) {
    this.setState({
      showRequestDetail: true,
      requestDetail: item
    });
  }
}

function parseData (data) {
  let result = [];

  for (let key in data) {
    let keyLabel = key.replace(/^\w|-\w/g, match => match.toUpperCase());
    result.push(<li key={key}><strong>{keyLabel}</strong>: {data[key]}</li>);
  }

  return result;
}

function getQueryObjFromURL (params) {
  if (!params) {
    return null;
  }

  let res = {};
  let fields = params.split('&');

  fields.forEach((field) => {
    let kv = field.split('=');
    let key = kv[0];
    let value = kv.slice(1).join('=');

    if (key) {
      if (key in res) {
        res[key] = [].concat(res[key], value);
      } else {
        res[key] = decodeURIComponent(value);
      }
    }
  });

  return res;
}

function getBody (t) {
  let body = t.body;
  let querystring = t.querystring;
  let data = null;
  let method = t.method.toLocaleLowerCase();
  let contentType = t.headers['content-type'] || '';
  let isJSON = contentType.indexOf('json') !== -1;

  if (method === 'get') {
    data = getQueryObjFromURL(querystring);
  } else if (method === 'post') {
    if (isJSON) {
      try {
        data = JSON.parse(body);
      } catch (e) {
        isJSON = false;
        data = getQueryObjFromURL(body);
      }
    } else {
      data = getQueryObjFromURL(body);
    }
  }

  return isJSON ? JSON.stringify(data, null, 2) : parseData(data);
}

function getBodyType (ctx) {
  let method = ctx.method.toLocaleLowerCase();
  if (method === 'get') {
    return 'Query String Paramenters';
  }

  if (method === 'post') {
    let isJson = false;
    try {
      JSON.parse(ctx.body);
      isJson = true;
    } catch (e) {}

    return isJson ? 'Request Payload' : 'Form data';
  }

  return null;
}

function cacheBody (id, val) {
  sessionStorage.setItem(id, val);
}

function getCacheBody (id) {
  return sessionStorage.getItem(id);
}
